/********************************************************
			BOOTLOADER HEADER FILE
	- All parameters must be set here to customize to 
	  actual hardware
***********************************************************/
/********************************************
		DEFINITIONS
	- Entry points for user app. 
********************************************/

	#define UserEntryAddress 0x0200
	#define UserHInt UserEntryAddress + 4	

/* Header Test Switches */

// Config bytes not possible with test version
//#pragma config =    0b00100110001100
//#pragma config+2 = 0b00110110000011
//#pragma CDATA_END

/*******************************************
		CLOCKING PARAMETERS
 - Change processor clock here.
 - Support is at Khz here, no more decimal. 
********************************************/

// Clocking
#define CPUCLK					4L	// in Mhz
#define CYCLK					CPUCLK / 4000L	// Instruction cycle (cpu div by 4)

// process value of USART BRG VALUE, BRGH = 1, BRG16 = 1
#define BAUDR			19200			// value of baud rate needed
#define CONSTBR1		CPUCLK * 1000000
#define CONSTBR2		CONSTBR1 / 4
#define CONSTBR3		CONSTBR2 / BAUDR
#define BAUDGEN			CONSTBR3 - 1

#if (BAUDGEN) > (255)
	#error Baud rate value not possible with BRDH=1  
#endif
/* **************** FLASH PROGRAM CAPACITY ***************/
// capacity of pgm memory in bytes (2 times number of 14bits words)
// 16 for 8k pgm words
#define FLASHCAPACITY 32		// ex 16 k for 16f876

/********************************************
		GLOBAL VARIABLES
********************************************/
#pragma rambank 0
bit boot_OK, boot_Verif;
char rxdata, bi, boot_tp;
// dump values sent to pc in case or error
char dmpadrh,dmpadrl,dmpdathrd,dmpdatlrd,dmpdathwr,dmpdatlwr;
#pragma rambank 1
char boot_bufflash[64];
#pragma rambank 0


/********************************************
		HARDWARE DEFINITIONS
********************************************/
// Define the switch port that will trigger bootloading
// switch activation must be 0 if switch makes 0 when pressed
// otherwise must be 1
#define SWITCH PORTA.5
#define TrisSw TRISA.5
#define SwitchActivation 0

// If a led is available for status info, define it here
#define LED LATA.4
#define TrisLed TRISA.4 = 0
#define LEDON 1;
#define LEDOFF 0;

	
/***********************************************
		PROTOTYPES
*************************************************/

void usermain (void);
void UserHighIntServ (void);
void UserLowIntServ (void);
void userreset (void);
