/**********************************************************
		RS232 SERIAL BOOTLOADER
 
 >>>>>>>>>>>>>>>>>>> SPECIFIC 16F EXTENDED SERIES ><<<<<<<<<	
  - All polled, no interrupt is used in this soft.
 - All parameters must be set in the corresponding .h file.
 
 ===> after compil copy resulting asm into bo.asm.
 	  then recompile in assembly after replacement 
 	  of CALL to user soft by a GOTO usermain 
************************************************************/

#include "Srl_Bootldr.h"

/****** V E C T O R  R E M A P P I N G **************************/

  /* -----------------------------------------------------------------
            INTERRUPTS HANDLER                
 	- interrups not used for bootloader
 	- for live soft, a simple jump to user interrupt handler
 --------------------------------------------------------------------- */

#pragma origin 0x004


void hrdw_int_high( void)
{
#asm
	CALL UserHighIntServ		// call chgd in goto by compiler
#endasm
}

/*------------- End Interrupts services --------------*/

/********************************************************
     BOOTLOADER ROUTINES
 **********************************************************/


/* ********** FLASH WRITE P16F*********** */
#pragma rambank -
void boot_writeflash (uns16 fadr) // 64 bytes at once in bufflash
 {
	char i,j,k,l,m,n,pointr;
	uns16 svfadr;
	uns16 locadr;
	
	svfadr = fadr;
	locadr = fadr;
	boot_OK = 1;
	GIE = 0;		// should be already at 0, for standardizi	
	EEIE = 0;
	if (fadr >= (UserEntryAddress))	// protect booloader zone
	{
		// *** first erase block
		EEADRH = locadr.high8;
		EEADRL = locadr.low8;
		EEIF = 0;
		CFGS = 0;
		EEPGD = 1;
		FREE = 1;
		WREN = 1;
		EECON2 = 0x55;
		EECON2 = 0xAA;
		WR = 1;
		nop();
		nop();
		FREE = 0;
		
		//  ***  then write data
		EEADRH = locadr.high8;
		EEADRL = locadr.low8;
		EEPGD = 1;
		CFGS = 0;
		WREN = 1;
		LWLO = 1;	// select latches
		EEIF =0;
		// write latches loop
		for (k = 0; k != 32; k++)
		{
			EEIF = 0;
			svfadr = locadr + k;
			pointr = k * 2;
			boot_bufflash[pointr+1] &= 0x3F;
			EEDATL = boot_bufflash[pointr];
			EEDATH = boot_bufflash[pointr+1];
			EECON2 = 0x55;
			EECON2 = 0xAA;
			WR = 1;
			nop();
			nop();
			EEADRL++;
		}
			// write flash now
		EEIF = 0;
		LWLO = 0;		// select flash
		EEADRL = locadr.low8;
		EEDATL = boot_bufflash[0];
		EEDATH = boot_bufflash[1];		
		EECON2 = 0x55;
		EECON2 = 0xAA;
		WR = 1;
		nop();
		nop();
		nop();
		nop();
		nop();
		nop();
		nop();
		nop();
		WREN = 0;
		LED = !LED;
	}
	else boot_OK = 1;
}

bit verif (uns16 fadr)	
{
	char i,j,k,l,m,n,pointr;
	uns16 svfadr;
	uns16 locadr;
	
	boot_OK = 1;
// Method 2 read flash inprogramming mode
	if (fadr >= (UserEntryAddress))	// ignore booloader zone
	{
		EEIF = 0;
		CFGS = 0;
		EEPGD = 1;
		for (n = 0; n != 32; n++)
		{
			svfadr = fadr + n;
			EEIF = 0;
			CFGS = 0;
			EEPGD = 1;
			EEADRL = svfadr.low8;
			EEADRH = svfadr.high8;
			pointr = n * 2;		
			RD = 1;
			nop();
			nop();
			nop();
			nop();
			k = EEDATL;
			j = EEDATH;
			j = j & 0x3F;
			l = boot_bufflash[pointr] & 0x3F;
			m = boot_bufflash[pointr+1];
			if (j !=l)	boot_OK = 0;
			if (k != m) boot_OK = 0;
		}
	}
	return boot_OK;
}
	
#pragma rambank -
//  *** serial port routines
void boot_outsrl (char outcar)
	{
	while (TRMT == 0);
	TXREG = outcar;
	}

char boot_waitsrl (void)
{
while (RCIF == 0)
	{nop();
	nop();
	}
return RCREG;
}

#pragma rambank 0
void bootprocess (void)
{
// *** main bootloader

	char rcvdata, i, j, BCC; 
	uns16 boot_fadr,btloop;
	char tp;
	bit OutRng;
	char kflash;

	GIE = 0;		// no interrupt, polling only
/* host synch*/
	bi = 0;
	OutRng = 0;
	boot_tp=0;
	kflash = FLASHCAPACITY / 2;
	while (bi == 0)
	{
				rxdata = boot_waitsrl();
		if (rxdata == 'R')
		{
			nop();
			nop();
			rxdata = boot_waitsrl();

			if (rxdata == 'M')
			{
				nop();
				nop();
				rxdata = boot_waitsrl();
				if (rxdata == 'b')
				{
					nop();
					nop();
					rxdata = boot_waitsrl();
					if (rxdata == 't')
					{
						LED = LEDOFF;
						boot_outsrl ('b');			// Device reply with c to complete handshake
						boot_outsrl ('t');
						boot_outsrl (FLASHCAPACITY);
						bi = 1;
					}	
				}
			}
		} 
	}

/* tfr data */
	boot_fadr = 0;

// transfer format is :
// 27 as sart of block
// address high, address low, in hex.
// then 64 bytes follows, then check caracter
// When address is 0xff00: stop

	while (boot_fadr != 0xFF00)	// must complete with address FF00 to stop tfr
	{
		rxdata = boot_waitsrl();
		while (rxdata != 27)		// start of block 
		{
			if (rxdata == 28)	// verif loop
			{
				if (verif (boot_fadr) == 1)
				{
					boot_outsrl (6);
				}
				else
				{
					boot_outsrl(8);		// for test, must be 8
				}		
			}
			rxdata = boot_waitsrl();
		}
		rcvdata = boot_waitsrl();
		BCC = rcvdata;
		boot_fadr.high8 = rcvdata;
		rcvdata = boot_waitsrl();
		BCC += rcvdata;
		boot_fadr.low8 = rcvdata;
		if (boot_fadr == 0xFF00) break;
		for (i = 0;i!=64;i++)
		{
			rcvdata = boot_waitsrl();
			BCC += rcvdata;
			boot_bufflash[i] = rcvdata;
		}
#ifdef LED
		boot_tp++;
		if (boot_tp == 5)
		{
			LED = !LED;
			boot_tp=0;
		}
#endif	
		rcvdata = boot_waitsrl();	// BCC caracter
		if (rcvdata == BCC)  
		{
			boot_OK=1;
			if (OutRng == 0)
			{		
				boot_writeflash (boot_fadr);
				if (boot_OK == 1)
				{
					boot_outsrl (6);		// check is ok, reply ack
				}
				else
				{
					boot_outsrl (8);		// reply 8 for error write
					boot_outsrl (dmpadrh);
					boot_outsrl(dmpadrl);
					boot_outsrl(dmpdathrd);
					boot_outsrl(dmpdatlrd);
					boot_outsrl(dmpdathwr);
					boot_outsrl(dmpdatlwr);

				}
			}
			else 
			{
				boot_outsrl(9);
//				boot_fadr = 0xFF00;
			}			
	
		}	
		else
		{
				boot_outsrl (7);	// reply nack, comm error
		}
	}
}

void Initialize()
{
	INTCON = 0;		//  disable all interrupts /
	STATUS = 0;
	PORTA = 0; 
	PORTC = 0;  
	TRISA = bin(111111);
	TRISC = bin(11111111);

#ifdef LED
	TrisLed;
#endif
	/* init periph interrupts */
	PIE1 = bin(00000000); /* disabled int adc, usart, all polled */
	PIR1 = 0;
	GIE = 0;			// no interrupt, polling hrdw
	clearRAM();
	/* init USART 19.2 Kbps / 8 bits*/
	//config USART
	TXSTA = 0b00100110;
	RCSTA = 0b10010000;
	BAUDCON = 0b00001000;
	SPBRGL = BAUDGEN;
	
	SPBRGH = 0;
	#if (BAUDGEN) > (255)
	SPBRGH = BAUDGEN / 256;
	#endif
	TRISC.4 = 1;
	TRISC.5 = 1;
	nop();
	bi = RCREG;		// dummy read to clear buffer
	nop();
	bi = RCREG;
}	


/************* RESET ENTRY POINT ***************/
void main( void)
{
	OSCCON = 0b01101010;	// 4MHz intnl oscilator
	OSCSTAT = 1;
	ADCON0 = 0;		// All analog ports as digital
	ADCON1 = 0x07;
	ANSELA = 0;
	ANSELC = 0;
// next line not needed, just in case wrong manip made on previous lines
	TrisSw = 1;		// input of course	
// test if switch pressed, otherwise jump to user code
	if (SWITCH == SwitchActivation)
	{
		Initialize();
#ifdef LED
		LED = LEDON;
#endif
		bootprocess();
	}
// if switch not pressed goto user startup

// CC5X does not allow a goto here below
// only way to change to GOTO is to use assembler later

	#asm
		CALL usermain		// call should be chgd in goto in asm
	#endasm

}

/* ---------------------------------------------------------
              DUMMY MAIN        
 	- This part is only here to generate the necessary
 	  adresses for the direct calls                           
 ---------------------------------------------------------*/
#pragma origin UserEntryAddress
void usermain( void)
{
// user code should start like the following code :

#asm
	call userreset
#endasm

}

#pragma origin UserHInt

void UserHighIntServ (void)
{
}

//  small example of structure of user code :	
void MainLoop (void)
{
}

void SystemInit (void)
{
}

void userreset ( void)
{
 	SystemInit();
	while (1)
	{
		MainLoop();
	}	

}


