/**********************************************************
		RS232 SERIAL BOOTLOADER
 
 - All polled, no interrupt is used in this soft.
 - All parameters must be set in the corresponding .h file.
************************************************************/

#include "int18XXX.H"
#include "Srl_Bootldr.h"


/****** V E C T O R  R E M A P P I N G **************************/

  /* -----------------------------------------------------------------
            INTERRUPTS HANDLER                
 	- interrups not used for bootloader
 	- for live soft, a simple jump to user interrupt handler
 --------------------------------------------------------------------- */

#pragma origin 0x008

void hrdw_int_high( void)
{
	#asm
	call UserHighIntServ		// call chgd in goto
	#endasm
}

#pragma origin 0x018
void hrdw_int_low(void)
{
	#asm
	call UserLowIntServ		// call chgd in goto
	#endasm
}

/*------------- End Interrupts services --------------*/

/********************************************************
     BOOTLOADER ROUTINES
 **********************************************************/


void boot_writeflash (uns16 fadr) // 64 bytes at once in bufflash
{
	char i,j,calc8,pointr;
	uns16 svfadr, tpadrs;
	
	boot_OK = 1;
	svfadr = fadr;
// erase row
	if (fadr >= 0x400)	// value must be above bootldr
	{
		TBLPTRU = 0;
		TBLPTRH = fadr.high8;
		TBLPTRL = fadr.low8;
		EEPGD = 1;
		CFGS = 0;
		WREN = 1;
		FREE = 1;
		GIE = 0;
		EECON2 = 0x55;
		EECON2 = 0xAA;
		WR = 1;
		nop();
		nop();
		while (FREE == 1);
		EEIF = 0;
		WREN = 0;
// Write datas
		tpadrs = fadr - 8;
		for (calc8 = 0;calc8 != 8;calc8++)
		{
			TBLPTRU = 0;
			TBLPTRH = fadr.high8;
			TBLPTRL = fadr.low8;
			tableReadDec(); // dummy read to adjust TBLPTR
			j = (char) calc8 * 8;
			for (i = 0;i != 8;i++)
			{
				pointr = j + i;
				TABLAT = boot_bufflash[pointr];
				tableWritePreInc();
			}
			EEPGD=1;
			CFGS = 0;
			WREN = 1;
			GIE = 0;
			EECON2 = 0x55;
			EECON2 = 0xAA;
			WR = 1;
			nop ();
			nop();
			EEIF = 0;
			WREN = 0;
			fadr = fadr + 8;
		}
		WREN = 0;
// Verify block just writen
		fadr = svfadr;
		TBLPTRU = 0;
		TBLPTRH = svfadr.high8;
		TBLPTRL = svfadr.low8;
		for(i = 0; i != 64; i++)
		{			
			tableReadInc();
			calc8=TABLAT;
			j = boot_bufflash[i];
			if (j != calc8)
			{
				boot_OK = 0;
				j = calc8;
				break;
			}
			if (boot_OK == 0) break;
		}
		if (boot_OK == 0)
		{
			dmpadrh = svfadr.high8;
			dmpadrl = svfadr.low8;
			dmpdathrd = 0;
			dmpdatlrd = j;
			dmpdathwr = boot_bufflash[pointr+1];
			dmpdatlwr = boot_bufflash[pointr];
		}	
	}
}

//  *** serial port routines
void boot_outsrl (char outcar)
	{
	while (TRMT == 0);
	#if defined(_18F25K50) || (_18F45K50)
	TXREG1 = outcar;
	#else
	TXREG = outcar;
	#endif
	}

char boot_waitsrl (void)
{
while (RCIF == 0)
	{nop();
	nop();
	}
RCIF = 0;
#if defined(_18F25K50) || (_18F45K50)
return RCREG1;
#else
return RCREG;
#endif
}

void bootprocess (void)
{
// *** main bootloader

	char rcvdata, i, j, BCC; 
	uns16 boot_fadr,btloop;
	GIE = 0;
	char tp;

/* host synch*/
	bi = 0;
	boot_tp=0;
	while (bi == 0)
	{
		rxdata = boot_waitsrl();
		if (rxdata == 'R')
		{
			nop();
			nop();
			rxdata = boot_waitsrl();

			if (rxdata == 'M')
			{
				nop();
				nop();
				rxdata = boot_waitsrl();
				if (rxdata == 'b')
				{
					nop();
					nop();
					rxdata = boot_waitsrl();
					if (rxdata == 't')
					{
						LED = LEDOFF;
						boot_outsrl ('b');			// Device reply with c to complete handshake
						boot_outsrl ('t');
						boot_outsrl (FLASHCAPACITY);
						bi = 1;
					}	
				}
			}
		} 
	}

/* tfr data */
	boot_fadr = 0;

// transfer format is :
// 27 as sart of block
// address high, address low, in hex.
// then 64 bytes follows, then check caracter
// When address is 0xff00: stop

	while (boot_fadr != 0xFF00)	// must complete with address FF00 to stop tfr
	{
		while (boot_waitsrl() != 27);		// start of block 
		rcvdata = boot_waitsrl();
		BCC = rcvdata;
		boot_fadr.high8 = rcvdata;
		rcvdata = boot_waitsrl();
		BCC += rcvdata;
		boot_fadr.low8 = rcvdata;
		if (boot_fadr == 0xFF00) break;
		for (i = 0;i!=64;i++)
		{
			rcvdata = boot_waitsrl();
			BCC += rcvdata;
			boot_bufflash[i] = rcvdata;
		}
#ifdef LED
		boot_tp++;
		if (boot_tp == 5)
		{
			LED = !LED;
			boot_tp=0;
		}
#endif	
		rcvdata = boot_waitsrl();	// BCC caracter
		if (rcvdata == BCC)  
		{
			boot_OK=1;		
			boot_writeflash (boot_fadr);
				if (boot_OK == 1)
				{
					boot_outsrl (6);		// write is ok, reply ack
				}
				else
				{						// In case of error send some infos
					boot_outsrl (8);
					boot_outsrl (dmpadrh);
					boot_outsrl(dmpadrl);
					boot_outsrl(dmpdathrd);
					boot_outsrl(dmpdatlrd);
					boot_outsrl(dmpdathwr);
					boot_outsrl(dmpdatlwr);
				}
	
		}
		else
		{
				boot_outsrl (7);	// reply nack (comm error)
		}
	}
}

void Initialize()
{
	#if defined(_18F25K50) || (_18F45K50)
	OSCTUNE = 0x80; //3X PLL ratio mode selected
	OSCCON = 0x70;  //Switch to 16MHz HFINTOSC
	OSCCON2 = 0x10; //Enable PLL, SOSC, PRI OSC drivers turned off
	while(PLLRDY != 1);   //Wait for PLL lock
	ACTCON = 0x80;  //
		// all ports as digital at reset
	ANSELA = 0;
	ANSELB = 0;
	ANSELC = 0;
	ANSELD = 0;
	ANSELE = 0;
	// turn power off on all periph, must be indivually turned on
	PMD0 = 0xFF;
	PMD1 = 0xFF;
	PMD0.6 = 0;		//USART ena
	#endif
	INTCON = 0;  /* disable all interrupts */
	STATUS = 0;
	WDTCON = 0;
	PORTA = 0; 
	PORTB = 0;
	PORTC = 0;  
	TRISA = bin(111111);
	TRISB = bin(11111111);
	TRISC = bin(11111111);
//	TRISD = 0b11111111;			// not valid for 28pins
	TrisLed
// next line not needed, just in case wrong manip made on previous lines
	GIE = 0;			// no interrupt, polling hrdw
	clearRAM();
	/* init USART 19.2 Kbps / 8 bits*/



#if defined(_18F25K50) || (_18F45K50)
	TXSTA1 = 0b00100000;
	RCSTA1 = 0b10010000;
	BAUDCON1 = 0b01000000;
	SYNC = 0;
	BRGH = 0;
	BRG16 = 0;
	SPBRGH1 = 0;
	SPBRG1 = 38;
#else
	TXSTA = 0b00100000;
	RCSTA = 0b10010000;
	BRGH = 1;			//comment in case of baud error
	SPBRG = (char)BAUDGEN;
#endif
#if defined(_18F2550) || (_18F4550)
	BAUDCON = 0b01000000;
	SYNC = 0;
	BRGH = 0;
	BRG16 = 0;
	SPBRGH = 0;
	SPBRG = 38;	
#endif
	
#if defined(_18F25K50) || (_18F45K50)
	bi = RCREG1;	// dummy reads to clear buffer.
	nop();
	bi = RCREG1;
#else
	bi = RCREG;	// dummy reads to clear buffer.
	nop();
	bi = RCREG;
#endif
}	

void jumpusermain (void)
{

	STKPTR = 0;			// reset stack pointer, clear call to this proc
	#asm
		call usermain		// call chgd in goto
	#endasm
}	

/************* RESET ENTRY POINT ***************/
void main( void)
{
	Initialize();
#ifdef LED
		LED = LEDOFF;
#endif
	ADCON0 = 0;
	ADCON1 = 7;
	TrisSw = 0;
// test if switch pressed, otherwise jump to user code
	if (SWITCH == SwitchActivation)
	{
#ifdef LED
		LED = LEDON;
#endif
		bootprocess();
	}
// if switch not pressed goto user startup
	jumpusermain ();

}

/* ---------------------------------------------------------
              DUMMY MAIN        
 	- This part is only here to generate the necessary
 	  adresses for the direct calls                           
 ---------------------------------------------------------*/

#pragma origin UserEntryAddress
void usermain( void)
{
// user code should start like the following code :
#asm
	call userreset
#endasm
}

#pragma origin UserHInt

void UserHighIntServ (void)
{
}
#pragma origin UserLInt
void UserLowIntServ (void)
{
}

//  small example of structure of user code :	
void MainLoop (void)
{
}

void SystemInit (void)
{
}		
void userreset ( void)
{

 	SystemInit();
	while (1)
	{
		TRISB.2=0;
		PORTB.2 = 1;
		MainLoop();
	}	
}
