
/* ------------------------------------------------*/
/*             BOOT LOADER                         */
/*-------------------------------------------------*/


/********** Boot code ***********/
char rxdata, bi;
char b_bufflash[64];
bit b_notOK;


/* ********** FLASH WRITE ************* */


void b_writeflash (uns16 fadr) // 64 bytes at once in bufflash
{
	char i,j,calc8,pointr;
	uns16 svfadr;
	
	b_notOK = 0;
	svfadr = fadr;
// erase row
	if ( (fadr < (0x7800-64)) )	// value must match the org of bootloader
	{
		TBLPTRU = 0;
		TBLPTRH = fadr.high8;
		TBLPTRL = fadr.low8;
		EEPGD = 1;
		CFGS = 0;
		WREN = 1;
		FREE = 1;
		GIE = 0;
		EECON2 = 0x55;
		EECON2 = 0xAA;
		WR = 1;
		nop();
		nop();
		while (FREE == 1);
		while (EEIF == 0);
		EEIF = 0;
		WREN = 0;
// Write datas
		for (calc8 = 0;calc8 != 8;calc8++)
		{
			TBLPTRH = fadr.high8;
			TBLPTRL = fadr.low8;
			tableReadDec(); // dummy read to adjust TBLPTR
			j = (char) calc8 * 8;
			for (i = 0;i != 8;i++)
			{
				pointr = j + i;
				TABLAT = b_bufflash[pointr];
				tableWritePreInc();
			}
			EEPGD=1;
			CFGS = 0;
			WREN = 1;
			GIE = 0;
			EECON2 = 0x55;
			EECON2 = 0xAA;
			WR = 1;
			nop ();
			while (EEIF == 0);
			nop();
			nop();
			EEIF = 0;
			WREN = 0;
			fadr = fadr + 8;
		}
		WREN = 0;
// Verify block just writen
		fadr = svfadr;
		TBLPTRU = 0;
		TBLPTRH = svfadr.high8;
		TBLPTRL = svfadr.low8;
		for(i = 0; i != 64; i++)
		{			
			tableReadInc();
			calc8=TABLAT;
			j = b_bufflash[i];
			if (j != calc8)
			{
				b_notOK = 1;
			}
		}
	}
}


void b_outsrl (char outcar)
	{
	while (TRMT == 0);
	TXREG = outcar;
	}

char b_waitsrl (void)
{
while (RCIF == 0)
	{nop();
	nop();
	}
return RCREG;
}

void BootLoad (void)
{
	char rcvdata, i, j, BCC; 
	uns16 b_fadr;
	b_resetinit();
	GIE = 0;

/* host synch*/
	bi = 0;
	while (bi == 0)
	{
		LED = 1;
		rxdata = b_waitsrl();
		if (rxdata == 'C')
		{
			nop();
			nop();
			rxdata = b_waitsrl();
			if (rxdata == 'D')
			{
				nop();
				nop();
				b_outsrl ('c');			// Device reply with c to complete handshake
				bi = 1;
			}
		} 
	}

/* tfr data */
		b_fadr = 0;

// transfer format is :
// address high, address low, in hex. When address is FF00, stops
// then 64 bytes follows. 

	while (b_fadr != 0xFF00)	// must complete with address FF00 to stop tfr
	{
		while (b_waitsrl() != 27);		// start of block 
		rcvdata = b_waitsrl();
		BCC = rcvdata;
		b_fadr.high8 = rcvdata;
		rcvdata = b_waitsrl();
		BCC += rcvdata;
		b_fadr.low8 = rcvdata;
		if (b_fadr == 0xFF00) break;
		{for (i = 0;i!=64;i++)
			{
				rcvdata = b_waitsrl();
				BCC += rcvdata;
				b_bufflash[i] = rcvdata;
			}
			rcvdata = b_waitsrl();	// BCC caracter
			LED=!LED;
			
			if (rcvdata == BCC )  
			{
				if (b_fadr == 0)
				{
					b_bufflash[0]=0;
					b_bufflash[1]=239;
					b_bufflash[2]=60;
					b_bufflash[3]=240;
				}	
				b_writeflash (b_fadr);
				if (b_notOK)
				{	
					b_outsrl(8);
				}
				else
				{
					b_outsrl(6);
				}	
			}
			else
			{
				b_outsrl (7);
			} 	
		}
	}
	b_cleardisplay();
	for (bi = 0; bi!=10; bi++)
	{
		b_outdata (str003(bi));
	}
	LED = 0;
	while(1);
}
